chiark / gitweb /
make use of logging API wherever appropriate
[elogind.git] / job.c
diff --git a/job.c b/job.c
index 2f7912d262439d46f4668eb9566802345ff2f485..ec0baba5ee575417e7417d63c922ae999bf1611b 100644 (file)
--- a/job.c
+++ b/job.c
@@ -20,7 +20,7 @@ Job* job_new(Manager *m, JobType type, Name *name) {
         j->type = type;
         j->name = name;
 
/* We don't link it here, that's what job_link() is for */
 
         return j;
 }
@@ -37,23 +37,106 @@ void job_free(Job *j) {
                 hashmap_remove(j->manager->jobs, UINT32_TO_PTR(j->id));
         }
 
-        hashmap_remove(j->manager->jobs_to_add, j->name);
-        set_remove(j->manager->jobs_to_remove, j);
+        manager_transaction_delete_job(j->manager, j);
 
-        /* Free data and next 'smaller' objects */
         free(j);
 }
 
-void job_dump(Job *j, FILE*f) {
+JobDependency* job_dependency_new(Job *subject, Job *object, bool matters) {
+        JobDependency *l;
+
+        assert(object);
+
+        /* Adds a new job link, which encodes that the 'subject' job
+         * needs the 'object' job in some way. If 'subject' is NULL
+         * this means the 'anchor' job (i.e. the one the user
+         * explcitily asked for) is the requester. */
+
+        if (!(l = new0(JobDependency, 1)))
+                return NULL;
+
+        l->subject = subject;
+        l->object = object;
+        l->matters = matters;
+
+        if (subject) {
+                l->subject_next = subject->subject_list;
+                subject->subject_list = l;
+        } else {
+                l->subject_next = object->manager->transaction_anchor;
+                object->manager->transaction_anchor = l;
+        }
+
+        if (l->subject_next)
+                l->subject_next->subject_prev = l;
+        l->subject_prev = NULL;
+
+        if ((l->object_next = object->object_list))
+                l->object_next->object_prev = l;
+        l->object_prev = NULL;
+        object->object_list = l;
+
+        return l;
+}
+
+void job_dependency_free(JobDependency *l) {
+        assert(l);
+
+        if (l->subject_prev)
+                l->subject_prev->subject_next = l->subject_next;
+        else if (l->subject)
+                l->subject->subject_list = l->subject_next;
+        else
+                l->object->manager->transaction_anchor = l->subject_next;
+
+        if (l->subject_next)
+                l->subject_next->subject_prev = l->subject_prev;
+
+        if (l->object_prev)
+                l->object_prev->object_next = l->object_next;
+        else
+                l->object->object_list = l->object_next;
+
+        if (l->object_next)
+                l->object_next->object_prev = l->object_prev;
+
+        free(l);
+}
+
+void job_dependency_delete(Job *subject, Job *object, bool *matters) {
+        JobDependency *l;
+
+        assert(object);
+
+        for (l = object->object_list; l; l = l->object_next) {
+                assert(l->object == object);
+
+                if (l->subject == subject)
+                        break;
+        }
+
+        if (!l) {
+                if (matters)
+                        *matters = false;
+                return;
+        }
+
+        if (matters)
+                *matters = l->matters;
+
+        job_dependency_free(l);
+}
+
+void job_dump(Job *j, FILE*f, const char *prefix) {
 
         static const char* const job_type_table[_JOB_TYPE_MAX] = {
                 [JOB_START] = "start",
                 [JOB_STOP] = "stop",
                 [JOB_VERIFY_STARTED] = "verify-started",
                 [JOB_RELOAD] = "reload",
+                [JOB_RELOAD_OR_START] = "reload-or-start",
                 [JOB_RESTART] = "restart",
                 [JOB_TRY_RESTART] = "try-restart",
-                [JOB_RESTART_FINISH] = "restart-finish"
         };
 
         static const char* const job_state_table[_JOB_STATE_MAX] = {
@@ -65,9 +148,23 @@ void job_dump(Job *j, FILE*f) {
         assert(j);
         assert(f);
 
-        fprintf(f, "Job %u (%s) → %s in state %s\n",
-                j->id,
-                name_id(j->name),
-                job_type_table[j->type],
-                job_state_table[j->state]);
+        fprintf(f,
+                "%sJob %u:\n"
+                "%s\tAction: %s → %s\n"
+                "%s\tState: %s\n",
+                prefix, j->id,
+                prefix, name_id(j->name), job_type_table[j->type],
+                prefix, job_state_table[j->state]);
+}
+
+bool job_is_anchor(Job *j) {
+        JobDependency *l;
+
+        assert(j);
+
+        for (l = j->object_list; l; l = l->object_next)
+                if (!l->subject)
+                        return true;
+
+        return false;
 }
@@ -20,7 +20,7 @@ Job* job_new(Manager *m, JobType type, Name *name) {
         j->name = name;
 
-        /* We don't link it here, that's what job_link() is for */
+        /* We don't link it here, that's what job_dependency() is for */
 
         return j;